import type { Callback, DataSource, FormConf, FormField } from './type'
import http from '@/api'
import type { Options } from '@/components/Render/type'
import useMatch from './useMatch'
import { showFailToast } from 'vant'

const useForm = (formConf: FormConf, formData: Recordable) => {
  const observerList = ref<Callback[]>([])
  const allFields = computed<FormField[]>(() => {
    const fields: FormField[] = []
    const getFields = (fieldList: FormField[]) => {
      fieldList.forEach((field) => {
        fields.push(field)
        if (field.children) {
          getFields(field.children)
        }
      })
    }
    getFields(formConf.fields)
    return fields
  })
  const _formData = toRef(formData)
  const { matchesGroup } = useMatch(formData)
  provide('formParser', { formData: _formData })

  const addObserver = (callback: Callback) => {
    observerList.value.push(callback)
  }

  const delObserver = (callback: Callback) => {
    observerList.value = observerList.value.filter((item) => item !== callback)
  }

  const trigger = (prop: string) => {
    formConf.rules.forEach((rule) => {
      const { condition, actions } = rule
      const { conditions } = condition
      const isProp = conditions.some((rule) => rule.field === prop)
      if (isProp) {
        const isMatch = matchesGroup(condition)
        actions.forEach((action) => {
          const { field, trigger, value } = action
          const target = allFields.value.find((f) => f.id === field)
          if (target) {
            if (trigger === 'show') {
              target.hidden = !isMatch
            } else if (trigger === 'hide') {
              target.hidden = isMatch
            } else if (trigger === 'required') {
              target.required = isMatch
            } else if (trigger === 'notRequired') {
              target.required = !isMatch
            } else if (trigger === 'readonly') {
              target.readonly = isMatch
            } else if (trigger === 'enabled') {
              target.readonly = !isMatch
            }
          }
          if (trigger === 'set') {
            if (!isMatch) return
            if (value === 'empty') {
              formData[field] = Array.isArray(formData[field]) ? [] : null
            } else if (value.startsWith('{') && value.endsWith('}')) {
              const key = value.slice(1, -1)
              formData[field] = formData[key]
            } else {
              formData[field] = value
            }
          } else if (trigger === 'source') {
            if (isMatch) {
              const source = formConf.dataSource.find((item) => item.id === value)
              if (source) {
                triggerDataSource(source)
              }
            }
          }
        })
      }
    })
  }

  /**
   * 触发数据源
   * @param source
   */
  const triggerDataSource = async (source: DataSource) => {
    const params: Recordable = {}
    Object.keys(source.params).forEach((key) => {
      const value: string = source.params[key]
      if (/\{[^{}]+}/g.test(value)) {
        const prop = value.slice(1, -1)
        params[key] = formData[prop]
      }
    })
    const headers: Recordable = {}
    Object.keys(source.headers).forEach((key) => {
      const value: string = source.headers[key]
      if (/\{[^{}]+}/g.test(value)) {
        const prop = value.slice(1, -1)
        headers[key] = formData[prop]
      }
    })
    const data: Recordable = {}
    Object.keys(source.data).forEach((key) => {
      const value: string = source.data[key]
      if (/\{[^{}]+}/g.test(value)) {
        const prop = value.slice(1, -1)
        data[key] = formData[prop]
      }
    })
    const res = await http.request<any>({
      url: source.url,
      method: source.method,
      params: params,
      headers: headers,
      data: data
    })
    if (res.success) {
      let { data } = res
      const { dataKey, dataValue, dataPath } = source
      if (dataPath) {
        data = dataPath.split('.').reduce((pre, item) => pre[item], data)
      }
      if (Array.isArray(data)) {
        const options: Options[] = data.map((item) => {
          return {
            label: item[dataKey],
            value: item[dataValue]
          }
        })
        observerList.value.forEach((callback) => {
          callback(options)
        })
      }
    } else {
      showFailToast(res.message)
    }
  }

  const initFormData = (elementList: FormField[]) => {
    elementList.forEach((field) => {
      if (field.type === 'formItem') {
        if (field.value !== undefined) {
          if (formData[field.id] === undefined) {
            formData[field.id] = field.value
          }
        }
      }
      if (field.children) {
        initFormData(field.children)
      }
    })
  }
  onMounted(() => {
    formConf.dataSource.forEach((source) => {
      if (source.init) {
        triggerDataSource(source)
      }
    })
  })
  // 初始化表单数据
  initFormData(formConf.fields)
  Object.keys(formData).forEach((key) => {
    trigger(key)
  })
  return {
    observerList,
    addObserver,
    delObserver,
    initFormData,
    trigger,
    triggerDataSource
  }
}

export default useForm
